#
# Configuration file for using the XML library in GNOME applications
#
XML2_LIBDIR="-L/usr/i686-w64-mingw32/sys-root/mingw/lib"
XML2_LIBS="-lxml2 -lz -llzma   -liconv   "
XML2_INCLUDEDIR="-I/usr/i686-w64-mingw32/sys-root/mingw/include/libxml2"
MODULE_VERSION="xml2-2.9.4"

